@@ -23,6 +23,7 @@
 /* We really don't want to require X11 headers...  */
 
 #include "config.h"
+#include <qglobal.h>
 #if defined Q_WS_X11 && ! defined K_WS_QTONLY
 #include <X11/Xlib.h>
 #include <X11/X.h>
@@ -469,6 +470,7 @@ dcop_call(
   IceFlush(dcop_ice_conn);
 
   free(outputData);
+  outputData = NULL;
 
   if (IceConnectionStatus(dcop_ice_conn) != IceConnectAccepted) {
       fprintf(stderr, "dcop_call(): Connection not accepted\n");
@@ -494,10 +496,9 @@ dcop_call(
       status = IceProcessMessages(dcop_ice_conn, &waitInfo, &readyRet);
 
       if (status == IceProcessMessagesIOError) {
-	  fprintf(stderr, "dcop_call(): IceProcessMessagesIOError\n");
-	  IceCloseConnection(dcop_ice_conn);
-	  free(outputData);
-	  return False;
+        fprintf(stderr, "dcop_call(): IceProcessMessagesIOError\n");
+        IceCloseConnection(dcop_ice_conn);
+        return False;
       }
 
       fprintf(stderr, "dcop_call(): readyRet == %s\n", readyRet ? "True" : "False");
