@@ -14,7 +14,7 @@
          This code may be linked against any version of the Qt toolkit
          from Troll Tech, Norway. $
 
-   $Id: qconfigDB.h 226470 2003-05-17 22:13:03Z mueller $	 
+   $Id: qconfigDB.h 591629 2006-10-02 21:29:03Z adridg $	 
 */
 
 namespace std { }
@@ -199,7 +199,7 @@ public:
   bool save(QTextStream& file, int count); 
   /**
    * Get the value for the key as a string. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString& key, QCString& value) const;
@@ -226,7 +226,7 @@ public:
   // ---------------
   /**
    * Get the value for the key as a long integer. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, long&) const;
@@ -243,7 +243,7 @@ public:
    * This will probably be not fast, but this methods are not suited to save 
    * large amounts of data. For saving anything else than UNICODE strings,
    * no such conversion is needed.
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, QString&) const;
@@ -256,7 +256,7 @@ public:
   // ---------------
   /**
    * Get the value for the key as a double. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, double&) const;
@@ -269,7 +269,7 @@ public:
   // ---------------
   /**
    * Get the value for the key as a boolean value. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, bool&) const;
@@ -282,7 +282,7 @@ public:
   // ---------------
   /**
    * Get the value for the key as a list of strings. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, list<QCString>&) const;
@@ -295,7 +295,7 @@ public:
   // --------------
   /**
    * Get the value for the key as a QStrList. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, QStrList&) const;
@@ -310,7 +310,7 @@ public:
    * Get the value for the key as a QStringList. Beware of the difference -
    * a QStringList is a list of QString objects, while QStrList handles
    * char* like objects.
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, QStringList&) const;
@@ -323,7 +323,7 @@ public:
   // --------------
   /**
    * Get the value for the key as a list of long integers. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, list<long>&) const;
@@ -336,7 +336,7 @@ public:
   // --------------
   /**
    * Get the value for the key as a list of integers. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, list<int>&) const;
@@ -370,7 +370,7 @@ public:
   // --------------
   /**
    * Get the value for the key as a list of doubles. 
-   * \a key is the key to search for, \value is a reference to the object
+   * \a key is the key to search for, \a value is a reference to the object
    * the value for the key is assigned to.
    */
   bool get(const QCString&, list<double>&) const;
@@ -612,7 +612,7 @@ public:
   /**
    * Save the file. 
    * \a header will be the comment in the first line of the file.
-   * If \a force is \tt true, a file opened read-only will be switched
+   * If \a force is \c true, a file opened read-only will be switched
    * to read and write mode and back after saving. 
    * @see ::setFileName
    */
@@ -654,7 +654,7 @@ public:
   /**
    * Return a string describing the version.
    */
-  static const char* version() { return "2.0 $Revision: 226470 $"; }
+  static const char* version() { return "2.0 $Revision: 591629 $"; }
   /**
    * Check wether the given file is locked.
    * The method returns zero if not, a number > zero is the pid of the process
@@ -688,7 +688,7 @@ public:
   static void CleanLockFiles(int);
   /**
    * Lock the current file.
-   * Locking is done by creating a file <filename>.lock. 
+   * Locking is done by creating a file \<filename\> lock. 
    * QConfigDB-objects will reject opening a file for reading and 
    * writing if a lockfile for the  filename exists.
    */
