@@ -1,7 +1,7 @@
 /*
     This file is part of libkabc.
     Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
-    Copyright (c) 2004 Szombathelyi Gyrgy <gyurco@freemail.hu>
+    Copyright (c) 2004 Szombathelyi György <gyurco@freemail.hu>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -15,8 +15,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 
@@ -513,8 +513,6 @@ KIO::Job *ResourceLDAPKIO::loadFromCache()
     job = KIO::get( url, true, false );
     connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
       this, SLOT( data( KIO::Job*, const QByteArray& ) ) );
-    connect( job, SIGNAL( result( KIO::Job* ) ),
-      this, SLOT( syncLoadSaveResult( KIO::Job* ) ) );
   }
   return job;
 }
@@ -546,6 +544,8 @@ bool ResourceLDAPKIO::load()
 
   job = loadFromCache();    
   if ( job ) {
+    connect( job, SIGNAL( result( KIO::Job* ) ),
+      this, SLOT( syncLoadSaveResult( KIO::Job* ) ) );
     enter_loop();
   }
   if ( mErrorMsg.isEmpty() ) {
@@ -685,6 +685,19 @@ void ResourceLDAPKIO::data( KIO::Job *, const QByteArray &data )
   } while ( ret != LDIF::MoreData );
 }
 
+void ResourceLDAPKIO::loadCacheResult( KIO::Job *job )
+{
+  mErrorMsg = "";
+  d->mError = job->error();
+  if ( d->mError && d->mError != KIO::ERR_USER_CANCELED ) {
+    mErrorMsg = job->errorString();
+  }
+  if ( !mErrorMsg.isEmpty() )
+    emit loadingError( this, mErrorMsg );
+  else
+    emit loadingFinished( this );
+}
+
 void ResourceLDAPKIO::result( KIO::Job *job )
 {
   mErrorMsg = "";
@@ -701,14 +714,14 @@ void ResourceLDAPKIO::result( KIO::Job *job )
   KIO::Job *cjob;
   cjob = loadFromCache();
   if ( cjob ) {
-    enter_loop();
-    job = cjob;
+    connect( cjob, SIGNAL( result( KIO::Job* ) ),
+      this, SLOT( loadCacheResult( KIO::Job* ) ) );
+  } else {
+    if ( !mErrorMsg.isEmpty() )
+      emit loadingError( this, mErrorMsg );
+    else
+      emit loadingFinished( this );
   }
-
-  if ( !mErrorMsg.isEmpty() )
-    emit loadingError( this, mErrorMsg );
-  else
-    emit loadingFinished( this );
 }
 
 bool ResourceLDAPKIO::save( Ticket* )
