@@ -14,18 +14,16 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
-// $Id: kapplication.h,v 1.312 2004/07/22 16:50:04 waba Exp $
-
 #ifndef _KAPP_H
 #define _KAPP_H
 
 // Version macros. Never put this further down.
 #include "kdeversion.h"
-#include "kdemacros.h"
+#include "kdelibs_export.h"
 
 class KConfig;
 class KCharsets;
@@ -33,7 +31,7 @@ class DCOPClient;
 class DCOPObject;
 
 typedef unsigned long Atom;
-#ifdef Q_WS_QWS
+#if !defined(Q_WS_X11)
 typedef void Display;
 #endif
 
@@ -56,7 +54,8 @@ class KApplicationPrivate;
 * Controls and provides information to all KDE applications.
 *
 * Only one object of this class can be instantiated in a single app.
-* This instance is always accessible via the kApplication() method.
+* This instance is always accessible via the 'kapp' global variable.
+* See cut() for an example.
 *
 * This class provides the following services to all KDE applications.
 *
@@ -91,14 +90,22 @@ class KApplicationPrivate;
 *
 * @short Controls and provides information to all KDE applications.
 * @author Matthias Kalle Dalheimer <kalle@kde.org>
-* @version $Id: kapplication.h,v 1.312 2004/07/22 16:50:04 waba Exp $
 */
-class KApplication : public QApplication, public KInstance
+class KDECORE_EXPORT KApplication : public QApplication, public KInstance
 {
 
   Q_OBJECT
 public:
-  enum CaptionLayout { CaptionAppLast=1, CaptionAppFirst, CaptionNoApp };
+  /** Position of the caption (presumably in the application window's
+  *   title bar). This enum appears to be unused.
+  *
+  * @todo Find out if this is used anywhere.
+  */
+  enum CaptionLayout {
+    CaptionAppLast=1 /**< Display the application name last (before document name). */, 
+    CaptionAppFirst /**< Display the application name first. */ , 
+    CaptionNoApp  /**< Do not display application name at all. */
+  };
 
   /**
    * This constructor takes aboutData and command line
@@ -115,7 +122,7 @@ public:
    */
   KApplication( bool allowStyles=true, bool GUIenabled=true);
 
-#ifndef Q_WS_QWS
+#ifdef Q_WS_X11
   /**
    * Constructor. Parses command-line arguments. Use this constructor when you
    * you need to use a non-default visual or colormap. 
@@ -171,7 +178,7 @@ public:
 #endif
 
   /**
-   * @deprecated do not use it at all, it will make your application crash, use KCmdineArgs
+   * @deprecated do not use it at all, it will make your application crash, use KCmdLineArgs
    *
    * Constructor. Parses command-line arguments.
    *
@@ -352,14 +359,14 @@ public:
    */
     void propagateSessionManager();
 
-    /*
+    /**
      * Reimplemented for internal purposes, mainly the highlevel
      *  handling of session management with KSessionManaged.
      * @internal
      */
   void commitData( QSessionManager& sm );
 
-    /*
+    /**
      * Reimplemented for internal purposes, mainly the highlevel
      *  handling of session management with KSessionManaged.
      * @internal
@@ -527,6 +534,10 @@ public slots:
    */
   void invokeBrowser( const QString &url, const QCString& startup_id );
   // KDE4 merge with above with startup_id = ""
+  /**
+  * Invoke the standard browser. Uses a @p startup_id of "" (empty)
+  * and is otherwise the same as the above function.
+  */
   void invokeBrowser( const QString &url );
 
   /**
@@ -740,8 +751,6 @@ public:
 
   /**
    * Starts a program via kdeinit.
-   * There will be no application startup notification, so you should
-   * use it only for starting internal processes.
    *
    * program name and arguments are converted to according to the
    * local encoding and passed as is to kdeinit.
@@ -1033,6 +1042,16 @@ public:
    */
   QStringList authorizeControlModules(const QStringList &menuIds);
 
+  /**
+   * Returns the state of the currently pressed keyboard modifiers (e.g. shift, control, etc.)
+   * and mouse buttons, similarly to QKeyEvent::state() and QMouseEvent::state().
+   * You usually should simply use the information provided by QKeyEvent and QMouseEvent,
+   * but it can be useful to query for the status of the modifiers at another moment
+   * (e.g. some KDE apps do that upon a drop event).
+   * @return the keyboard modifiers and mouse buttons state
+   * @since 3.4
+   */
+  static ButtonState keyboardMouseState();
 
   // Same values as ShiftMask etc. in X.h
   enum { ShiftModifier = 1<<0,
@@ -1044,33 +1063,22 @@ public:
          Modifier4 = 1<<6,
          Modifier5 = 1<<7 };
   /**
-   * Returns the currently pressed keyboard modifiers (e.g. shift, control, etc.)
-   * Usually you simply want to test for those in key events, in which case
-   * QKeyEvent::state() does the job (or QKeyEvent::key() to
-   * notice when a modifier is pressed alone).
-   * But it can be useful to query for the status of the modifiers at another moment
-   * (e.g. some KDE apps do that upon a drop event).
-   * @return the keyboard modifiers
+   * @deprecated Use keyboardMouseState()
    * @since 3.1
    */
-  static uint keyboardModifiers();
+  static uint keyboardModifiers() KDE_DEPRECATED;
 
-  // Same values as Button1Mask etc. in X.h
+  /** @deprecated Same values as Button1Mask etc. in X.h */
   enum { Button1Pressed = 1<<8,
          Button2Pressed = 1<<9,
          Button3Pressed = 1<<10,
          Button4Pressed = 1<<11,
          Button5Pressed = 1<<12 };
   /**
-   * Returns the currently pressed mouse buttons.
-   * Usually you simply want to test for those in mouse events, in which case
-   * QMouseEvent::button() does the job (or QMouseEvent::state()).
-   * But it can be useful to query for the status of the mouse buttons at another moment.
-   * To query for the mouse pointer position, use QCursor::pos().
-   * @return the currently pressed mouse buttons
+   * @deprecated Use keyboardMouseState()
    * @since 3.1
    */
-  static uint mouseState();
+  static uint mouseState() KDE_DEPRECATED;
 
 
 public slots:
@@ -1161,8 +1169,13 @@ private:
   static bool s_dcopClientNeedsPostInit;
   QString aCaption; // the name for the window title
   bool bSessionManagement;
-  QPixmap aIconPixmap; // BIC (KDE 4) make a pointer, to avoid creating if noGUI
-  QPixmap aMiniIconPixmap; // BIC (KDE 4) make a pointer, to avoid creating if noGUI
+  struct oldPixmapType { QPixmap a, b; };
+  mutable union {
+    struct {
+      QPixmap *icon, *miniIcon;
+    } pm;
+    char unused[sizeof(oldPixmapType)];
+  } aIconPixmap; // KDE4: remove me
   QString aIconName;
   QString aMiniIconName;
   bool useStyles;
@@ -1363,6 +1376,12 @@ signals:
    */
   void shutDown();
 
+  /**
+   * @internal
+   * Used to notify KIconLoader objects that they need to reload.
+   */
+  void updateIconLoaders();
+
 private:
   void propagateSettings(SettingsCategory category);
   void kdisplaySetPalette();
@@ -1399,7 +1418,7 @@ private:
  * @param mode     The access mode, as in the access() system call.
  * @return Whether the access is allowed, true = Access allowed
  */
-bool checkAccess(const QString& pathname, int mode);
+KDECORE_EXPORT bool checkAccess(const QString& pathname, int mode);
 
 class KSessionManagedPrivate;
 
@@ -1419,7 +1438,7 @@ class KSessionManagedPrivate;
   @short Highlevel access to session management.
   @author Matthias Ettrich <ettrich@kde.org>
  */
-class KSessionManaged
+class KDECORE_EXPORT KSessionManaged
 {
 public:
   KSessionManaged();
