@@ -1,6 +1,6 @@
 /*
 
-   $Id: kprocess.cpp 503447 2006-01-29 09:42:40Z ossi $
+   $Id: kprocess.cpp 549079 2006-06-07 11:18:16Z mueller $
 
    This file is part of the KDE libraries
    Copyright (C) 1997 Christian Czezatke (e9025461@student.tuwien.ac.at)
@@ -355,8 +355,6 @@ bool KProcess::start(RunMode runmode, Communication comm)
   if (pipe(fd))
      fd[0] = fd[1] = -1; // Pipe failed.. continue
 
-  QApplication::flushX();
-
   // we don't use vfork() because
   // - it has unclear semantics and is not standardized
   // - we do way too much magic in the child
@@ -389,7 +387,10 @@ bool KProcess::start(RunMode runmode, Communication comm)
            if (pw)
               initgroups(pw->pw_name, pw->pw_gid);
 #endif
-           setuid(getuid());
+	   if (geteuid() != getuid())
+	       setuid(getuid());
+	   if (geteuid() != getuid())
+	       _exit(1);
         }
 
         setupEnvironment();
@@ -768,7 +769,7 @@ void KProcess::setUseShell(bool useShell, const char *shell)
     d->shell = shell;
   else
 // #ifdef NON_FREE // ... as they ship non-POSIX /bin/sh
-#if !defined(__linux__) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__GNU__)
+#if !defined(__linux__) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__GNU__) && !defined(__DragonFly__)
   // Solaris POSIX ...
   if (!access( "/usr/xpg4/bin/sh", X_OK ))
     d->shell = "/usr/xpg4/bin/sh";
