@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmcupsjobmanager.h,v 1.3 2001/05/22 14:48:08 goffioul Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -15,26 +13,41 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #ifndef KMCUPSJOBMANAGER_H
 #define KMCUPSJOBMANAGER_H
 
 #include "kmjobmanager.h"
+#include "ipprequest.h"
+#include <qstringlist.h>
+
+class KMPrinter;
 
 class KMCupsJobManager : public KMJobManager
 {
+	Q_OBJECT
+
 public:
-	KMCupsJobManager(QObject *parent = 0, const char *name = 0);
+	KMCupsJobManager(QObject *parent, const char *name, const QStringList &/*args*/);
 	virtual ~KMCupsJobManager();
 
 	int actions();
+	QValueList<KAction*> createPluginActions(KActionCollection*);
+	void validatePluginActions(KActionCollection*, const QPtrList<KMJob>&);
+	bool doPluginAction(int, const QPtrList<KMJob>&);
 
 protected:
-	bool listJobs();
-	bool sendCommandSystemJob(const QList<KMJob>& jobs, int action, const QString& arg = QString::null);
+	bool jobIppReport(KMJob*);
+	bool changePriority(const QPtrList<KMJob>&, bool);
+	bool editJobAttributes(KMJob*);
+
+protected:
+	bool listJobs(const QString&, JobType, int = 0);
+	bool sendCommandSystemJob(const QPtrList<KMJob>& jobs, int action, const QString& arg = QString::null);
+	void parseListAnswer(IppRequest& req, KMPrinter *pr);
 };
 
 #endif
