@@ -14,8 +14,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #include "kprinterimpl.h"
@@ -257,8 +257,8 @@ bool KPrinterImpl::startPrinting(const QString& cmd, KPrinter *printer, const QS
 	for (QStringList::ConstIterator it=files.begin(); it!=files.end(); ++it)
 		if (QFile::exists(*it))
 		{
-			// quote and encode filenames
-			filestr.append(quote(QFile::encodeName(*it))).append(" ");
+			// quote filenames
+			filestr.append(quote(*it)).append(" ");
 			printfiles.append(*it);
 		}
 		else
