@@ -1,8 +1,6 @@
 /*
  *  This file is part of the KDE libraries
- *  Copyright (c) 2001 Michael Goffioul <goffioul@imec.be>
- *
- *  $Id: kmpropdriver.h,v 1.2 2001/03/24 07:27:49 waba Exp $
+ *  Copyright (c) 2001 Michael Goffioul <kdeprint@swing.be>
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -15,8 +13,8 @@
  *
  *  You should have received a copy of the GNU Library General Public License
  *  along with this library; see the file COPYING.LIB.  If not, write to
- *  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- *  Boston, MA 02111-1307, USA.
+ *  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ *  Boston, MA 02110-1301, USA.
  **/
 
 #ifndef KMPROPDRIVER_H
@@ -24,9 +22,11 @@
 
 #include "kmpropwidget.h"
 
+#include <kdelibs_export.h>
+
 class QLabel;
 
-class KMPropDriver : public KMPropWidget
+class KDEPRINT_EXPORT KMPropDriver : public KMPropWidget
 {
 public:
 	KMPropDriver(QWidget *parent = 0, const char *name = 0);
