@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -126,8 +126,6 @@ class KAboutContainerBase : public QWidget
     void mailClick( const QString &name, const QString &address );
 
   private:
-    QMemArray<QWidget*>  mContainerList;
-
     QVBoxLayout *mTopLayout;
     KImageTrackLabel *mImageLabel;
     QLabel  *mTitleLabel;
