@@ -2,7 +2,7 @@
 
    Copyright (c) 2000,2001,2002 Carsten Pfeiffer <pfeiffer@kde.org>
    Copyright (c) 2000 Stefan Schimanski <1Stein@gmx.de>
-   Copyright (c) 2000,2001 Dawit Alemayehu <adawit@kde.org>
+   Copyright (c) 2000,2001,2002,2003,2004 Dawit Alemayehu <adawit@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -16,8 +16,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 
@@ -41,16 +41,18 @@ public:
     bool tabHandling;
     bool down_workaround;
     bool upwardBox;
+    bool emitSelected;
 };
 
 KCompletionBox::KCompletionBox( QWidget *parent, const char *name )
-    :KListBox( parent, name, WType_Popup )
+ :KListBox( parent, name, WType_Popup ), d(new KCompletionBoxPrivate)
 {
-    d = new KCompletionBoxPrivate;
+
     d->m_parent        = parent;
     d->tabHandling     = true;
     d->down_workaround = false;
     d->upwardBox       = false;
+    d->emitSelected    = true;
 
     setColumnMode( 1 );
     setLineWidth( 1 );
@@ -83,9 +85,14 @@ KCompletionBox::~KCompletionBox()
 QStringList KCompletionBox::items() const
 {
     QStringList list;
-    for ( uint i = 0; i < count(); i++ ) {
-        list.append( text( i ) );
+
+    const QListBoxItem* currItem = firstItem();
+
+    while (currItem) {
+        list.append(currItem->text());
+        currItem = currItem->next();
     }
+
     return list;
 }
 
@@ -127,17 +134,14 @@ bool KCompletionBox::eventFilter( QObject *o, QEvent *e )
                         ev->accept();
                         return true;
                     case Key_Up:
-
                         // If there is no selected item and we've popped up above
                         // our parent, select the first item when they press up.
-
-			if ( selectedItem() ||
+                        if ( selectedItem() ||
                              mapToGlobal( QPoint( 0, 0 ) ).y() >
                              d->m_parent->mapToGlobal( QPoint( 0, 0 ) ).y() )
                             up();
                         else
                             down();
-
                         ev->accept();
                         return true;
                     case Key_Prior:
@@ -229,6 +233,14 @@ bool KCompletionBox::eventFilter( QObject *o, QEvent *e )
         QMouseEvent *ev = static_cast<QMouseEvent *>( e );
         if ( !rect().contains( ev->pos() )) // this widget
             hide();
+
+        if ( !d->emitSelected && currentItem() && !::qt_cast<QScrollBar*>(o) )
+        {
+          emit highlighted( currentText() );
+          hide();
+          ev->accept();  // Consume the mouse click event...
+          return true;
+        }
     }
 
     return KListBox::eventFilter( o, e );
@@ -301,7 +313,7 @@ void KCompletionBox::show()
     // is in a layout, that layout will detect inserting new child (posted
     // ChildInserted event), and will trigger relayout (post LayoutHint event).
     // QWidget::show() sends also posted ChildInserted events for the parent,
-    // and later all LayoutHint events, which causes layout updating. 
+    // and later all LayoutHint events, which causes layout updating.
     // The problem is, KCompletionBox::eventFilter() detects resizing
     // of the parent, and calls hide() - and this hide() happen in the middle
     // of show(), causing inconsistent state. I'll try to submit a Qt patch too.
@@ -443,11 +455,11 @@ class KCompletionBoxItem : public QListBoxItem
 {
 public:
     //Returns true if dirty.
-    bool reuse( const QString& newText ) 
-    {   
+    bool reuse( const QString& newText )
+    {
         if ( text() == newText )
             return false;
-        setText( newText ); 
+        setText( newText );
         return true;
     }
 };
@@ -476,9 +488,13 @@ void KCompletionBox::setItems( const QStringList& items )
         //so we can avoid a repaint for identical updates,
         //to reduce flicker
         bool dirty = false;
-        for ( QStringList::ConstIterator it = items.begin(); it != items.end(); it++) {
+
+        QStringList::ConstIterator it = items.constBegin();
+        const QStringList::ConstIterator itEnd = items.constEnd();
+
+        for ( ; it != itEnd; ++it) {
             if ( item ) {
-                bool changed = ((KCompletionBoxItem*)item)->reuse( *it );
+                const bool changed = ((KCompletionBoxItem*)item)->reuse( *it );
                 dirty = dirty || changed;
                 item = item->next();
             }
@@ -488,10 +504,10 @@ void KCompletionBox::setItems( const QStringList& items )
                 insertItem( new QListBoxText( *it ) );
             }
         }
-        
+
         //If there is an unused item, mark as dirty -> less items now
         if ( item ) {
-            dirty = true; 
+            dirty = true;
         }
 
         QListBoxItem* tmp = item;
@@ -499,11 +515,14 @@ void KCompletionBox::setItems( const QStringList& items )
             tmp = item->next();
             delete item;
         }
-        
+
         if (dirty)
             triggerUpdate( false );
     }
 
+    if ( isVisible() && size().height() != sizeHint().height() )
+        sizeAndPosition();
+
     blockSignals( block );
     d->down_workaround = true;
 }
@@ -527,6 +546,16 @@ void KCompletionBox::slotItemClicked( QListBoxItem *item )
     }
 }
 
+void KCompletionBox::setActivateOnSelect(bool state)
+{
+    d->emitSelected = state;
+}
+
+bool KCompletionBox::activateOnSelect() const
+{
+    return d->emitSelected;
+}
+
 void KCompletionBox::virtual_hook( int id, void* data )
 { KListBox::virtual_hook( id, data ); }
 
