@@ -14,8 +14,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <qstringlist.h>
@@ -102,6 +102,7 @@ void KEditListBox::init( bool checkAtEntering, int buttons,
     grid->addMultiCellWidget(editingWidget,1,1,0,1);
     grid->addMultiCellWidget(m_listBox, 2, 6, 0, 0);
 
+    d->buttons = 0;
     setButtons( buttons );
 
     connect(m_lineEdit,SIGNAL(textChanged(const QString&)),this,SLOT(typedSomething(const QString&)));
