@@ -196,6 +196,9 @@ void HTMLLinkElementImpl::process()
                 getDocument()->addPendingSheet();
 
             QString chset = getAttribute( ATTR_CHARSET ).string();
+            // set chset to charset of referring document when attribute CHARSET is absent.
+            // http://www.w3.org/TR/CSS21/syndata.html(4.4)
+            if (chset.isEmpty() && part) chset = part->encoding();
             if (m_cachedSheet)
 		m_cachedSheet->deref(this);
             m_cachedSheet = getDocument()->docLoader()->requestStyleSheet(m_url, chset);
@@ -230,12 +233,13 @@ void HTMLLinkElementImpl::removedFromDocument()
     getDocument()->updateStyleSelector();
 }
 
-void HTMLLinkElementImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr)
+void HTMLLinkElementImpl::setStyleSheet(const DOM::DOMString &url, const DOM::DOMString &sheetStr, const DOM::DOMString &charset)
 {
     if (m_sheet)
         m_sheet->deref();
     m_sheet = new CSSStyleSheetImpl(this, url);
     m_sheet->ref();
+    m_sheet->setCharset(charset);
     m_sheet->parseString( sheetStr, !getDocument()->inCompatMode() );
 
     MediaListImpl *media = new MediaListImpl( m_sheet, m_media );
@@ -298,8 +302,6 @@ void HTMLMetaElementImpl::parseAttribute(AttributeImpl *attr)
 	m_content = attr->value();
 	process();
 	break;
-    case ATTR_NAME:
-      break;
     default:
         HTMLElementImpl::parseAttribute(attr);
     }
@@ -321,7 +323,7 @@ void HTMLMetaElementImpl::process()
 
 // -------------------------------------------------------------------------
 
-HTMLScriptElementImpl::HTMLScriptElementImpl(DocumentPtr *doc)
+HTMLScriptElementImpl::HTMLScriptElementImpl(DocumentImpl *doc)
     : HTMLElementImpl(doc), m_cachedScript(0), m_createdByParser(false), m_evaluated(false)
 {
 }
@@ -393,10 +395,12 @@ void HTMLScriptElementImpl::notifyFinished(CachedObject* o)
 
     assert(cs == m_cachedScript);
 
-    evaluateScript(cs->url().string(), cs->script());
-
+    QString   URL    = cs->url().string();
+    DOMString script = cs->script();
     cs->deref(this);
     m_cachedScript = 0;
+
+    evaluateScript(URL, script);
 }
 
 void HTMLScriptElementImpl::evaluateScript(const QString &URL, const DOMString &script)
