@@ -114,6 +114,26 @@ void RenderBox::restructureParentFlow() {
     }
 }
 
+static inline bool overflowAppliesTo(RenderObject* o) 
+{
+     // css 2.1-11.1.1
+     // 1) overflow only applies to non-replaced block-level elements, table cells, and inline-block elements
+     if (o->isRenderBlock() || o->isTableRow() || o->isTableSection())
+         // 2) overflow on root applies to the viewport (cf. KHTMLView::layout)
+         if (!o->isRoot())
+             // 3) overflow on body may apply to the viewport...
+             if (!o->isBody()
+                   // ...but only for HTML documents...
+                   || !o->document()->isHTMLDocument()
+                   // ...and only when the root has a visible overflow   
+                   || !o->document()->documentElement()->renderer()
+                   || !o->document()->documentElement()->renderer()->style()
+                   ||  o->document()->documentElement()->renderer()->style()->hidesOverflow())
+                 return true;
+
+     return false; 
+} 
+
 void RenderBox::setStyle(RenderStyle *_style)
 {
     bool affectsParent = style() && isFloatingOrPositioned() &&
@@ -155,6 +175,9 @@ void RenderBox::setStyle(RenderStyle *_style)
             setRelPositioned(true);
     }
 
+    if (overflowAppliesTo(this) && _style->hidesOverflow())
+        setHasOverflowClip();
+        
     if (requiresLayer()) {
         if (!m_layer) {
             m_layer = new (renderArena()) RenderLayer(this);
@@ -213,7 +236,7 @@ short RenderBox::contentWidth() const
     short w = m_width - style()->borderLeftWidth() - style()->borderRightWidth();
     w -= paddingLeft() + paddingRight();
 
-    if (m_layer && style()->scrollsOverflow())
+    if (m_layer && scrollsOverflowY())
         w -= m_layer->verticalScrollbarWidth();
 
     //kdDebug( 6040 ) << "RenderBox::contentWidth(2) = " << w << endl;
@@ -225,7 +248,7 @@ int RenderBox::contentHeight() const
     int h = m_height - style()->borderTopWidth() - style()->borderBottomWidth();
     h -= paddingTop() + paddingBottom();
 
-    if (m_layer && style()->scrollsOverflow())
+    if (m_layer && scrollsOverflowX())
         h -= m_layer->horizontalScrollbarHeight();
 
     return h;
@@ -295,7 +318,7 @@ void RenderBox::paint(PaintInfo& i, int _tx, int _ty)
     _tx += m_x;
     _ty += m_y;
 
-    if (style()->hidesOverflow() && m_layer)
+    if (hasOverflowClip() && m_layer)
         m_layer->subtractScrollOffset(_tx, _ty);
 
     // default implementation. Just pass things through to the children
@@ -568,24 +591,26 @@ void RenderBox::paintBackgroundExtended(QPainter *p, const QColor &c, const Back
             calculateBackgroundSize(bgLayer, scaledImageWidth, scaledImageHeight);
             EBackgroundRepeat bgr = bgLayer->backgroundRepeat();
 
-            if( (bgr == NO_REPEAT || bgr == REPEAT_Y) && pw > scaledImageWidth ) {
-                cw = scaledImageWidth;
-                cx = vr.x() + bgLayer->backgroundXPosition().minWidth(pw - scaledImageWidth);
+            int xPosition = bgLayer->backgroundXPosition().minWidth(pw-scaledImageWidth);
+            if (bgr == NO_REPEAT || bgr == REPEAT_Y) {
+                cw = kMin(scaledImageWidth, pw - xPosition);
+                cx = vr.x() + xPosition;
             } else {
                 cw = pw;
                 cx = vr.x();
                 if (scaledImageWidth > 0)
-                    sx = scaledImageWidth - bgLayer->backgroundXPosition().minWidth(pw - scaledImageWidth) % scaledImageWidth;
+                    sx = scaledImageWidth - xPosition % scaledImageWidth;
             }
 
-            if( (bgr == NO_REPEAT || bgr == REPEAT_X) && ph > scaledImageHeight ) {
-                ch = scaledImageHeight;
-                cy = vr.y() + bgLayer->backgroundYPosition().minWidth(ph - scaledImageHeight);
+            int yPosition = bgLayer->backgroundYPosition().minWidth(ph-scaledImageHeight);
+            if (bgr == NO_REPEAT || bgr == REPEAT_X) {
+                ch = kMin(scaledImageHeight, ph - yPosition);
+                cy = vr.y() + yPosition;
             } else {
                 ch = ph;
                 cy = vr.y();
                 if (scaledImageHeight > 0)
-                    sy = scaledImageHeight - bgLayer->backgroundYPosition().minWidth(ph - scaledImageHeight) % scaledImageHeight;
+                    sy = scaledImageHeight - yPosition % scaledImageHeight;
             }
 
             QRect fix(cx, cy, cw, ch);
@@ -705,6 +730,10 @@ short RenderBox::containingBlockWidth() const
     }
 
     RenderBlock* cb = containingBlock();
+    if (isRenderBlock() && cb->isTable() && static_cast<RenderTable*>(cb)->caption() == this) {
+        //captions are not affected by table border or padding
+        return cb->width();
+    }
     if (usesLineWidth())
         return cb->lineWidth(m_y);
     else
@@ -719,7 +748,7 @@ bool RenderBox::absolutePosition(int &_xPos, int &_yPos, bool f) const
     if( o && o->absolutePosition(_xPos, _yPos, f))
     {
         if ( o->layer() ) {
-            if (o->style()->hidesOverflow())
+            if (o->hasOverflowClip())
                 o->layer()->subtractScrollOffset( _xPos, _yPos );
             if (isPositioned())
                 o->layer()->checkInlineRelOffset(this, _xPos, _yPos);
@@ -805,7 +834,7 @@ void RenderBox::repaintRectangle(int x, int y, int w, int h, Priority p, bool f)
     RenderObject *o = container();
     if( o ) {
          if (o->layer()) {
-             if (o->style()->hidesOverflow())
+             if (o->style()->hidesOverflow() && o->layer() && !o->isInlineFlow())
                  o->layer()->subtractScrollOffset(x,y); // For overflow:auto/scroll/hidden.
              if (style()->position() == ABSOLUTE)
                  o->layer()->checkInlineRelOffset(this,x,y);
@@ -964,7 +993,7 @@ void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw
     }
     else
     {
-        if ( (ml.isVariable() && mr.isVariable()) ||
+        if ( (ml.isVariable() && mr.isVariable() && m_width<cw) ||
              (!ml.isVariable() && !mr.isVariable() &&
                 containingBlock()->style()->textAlign() == KHTML_CENTER) )
         {
@@ -972,14 +1001,14 @@ void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw
             if (m_marginLeft<0) m_marginLeft=0;
             m_marginRight = cw - m_width - m_marginLeft;
         }
-        else if (mr.isVariable() ||
+        else if ( (mr.isVariable() && m_width<cw) ||
                  (!ml.isVariable() && containingBlock()->style()->direction() == RTL &&
                   containingBlock()->style()->textAlign() == KHTML_LEFT))
         {
             m_marginLeft = ml.width(cw);
             m_marginRight = cw - m_width - m_marginLeft;
         }
-        else if (ml.isVariable() ||
+        else if ( (ml.isVariable() && m_width<cw) ||
                  (!mr.isVariable() && containingBlock()->style()->direction() == LTR &&
                   containingBlock()->style()->textAlign() == KHTML_RIGHT))
         {
@@ -988,6 +1017,7 @@ void RenderBox::calcHorizontalMargins(const Length& ml, const Length& mr, int cw
         }
         else
         {
+           // this makes auto margins 0 if we failed a m_width<cw test above (css2.1, 10.3.3)
             m_marginLeft = ml.minWidth(cw);
             m_marginRight = mr.minWidth(cw);
         }
@@ -1044,14 +1074,14 @@ void RenderBox::calcHeight()
             height = calcBoxHeight(h.value());
         }
 
-        if (height<m_height && !overhangingContents() && style()->overflow()==OVISIBLE)
+        if (height<m_height && !overhangingContents() && !hasOverflowClip())
             setOverhangingContents();
 
         m_height = height;
     }
 
     // Unfurling marquees override with the furled height.
-    if (style()->overflow() == OMARQUEE && m_layer && m_layer->marquee() &&
+    if (style()->overflowX() == OMARQUEE && m_layer && m_layer->marquee() &&
         m_layer->marquee()->isUnfurlMarquee() && !m_layer->marquee()->isHorizontal()) {
         m_layer->marquee()->setEnd(m_height);
         m_height = kMin(m_height, m_layer->marquee()->unfurlPos());
@@ -1709,10 +1739,6 @@ void RenderBox::calcAbsoluteVertical()
 
     height += bordersPlusPadding;
 
-    // If our natural/content height exceeds the new height once we've set it, then we
-    // need to make sure to update overflow to track the spillout.
-    if (m_height > height && isRenderBlock())
-        static_cast<RenderBlock*>(this)->setOverflowHeight(m_height);
     // Set final height value.
     m_height = height;
 }
