@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  */
 
@@ -31,6 +31,7 @@
 #include <signal.h>
 
 #include <kapplication.h>
+#include <kstandarddirs.h>
 #include <qimage.h>
 #include <qfile.h>
 #include "test_regression.h"
@@ -43,6 +44,21 @@
 #include "rendering/render_layer.h"
 #include "khtmldefaults.h"
 
+//We don't use the default fonts, though, but traditional testregression ones
+#undef HTML_DEFAULT_VIEW_FONT
+#undef HTML_DEFAULT_VIEW_FIXED_FONT
+#undef HTML_DEFAULT_VIEW_SERIF_FONT
+#undef HTML_DEFAULT_VIEW_SANSSERIF_FONT
+#undef HTML_DEFAULT_VIEW_CURSIVE_FONT
+#undef HTML_DEFAULT_VIEW_FANTASY_FONT
+#define HTML_DEFAULT_VIEW_FONT "helvetica"
+#define HTML_DEFAULT_VIEW_FIXED_FONT "courier"
+#define HTML_DEFAULT_VIEW_SERIF_FONT "times"
+#define HTML_DEFAULT_VIEW_SANSSERIF_FONT "helvetica"
+#define HTML_DEFAULT_VIEW_CURSIVE_FONT "helvetica"
+#define HTML_DEFAULT_VIEW_FANTASY_FONT "helvetica"
+
+
 #include <kaction.h>
 #include <kcmdlineargs.h>
 #include "khtml_factory.h"
@@ -63,6 +79,8 @@
 #include <qwidget.h>
 #include <qfileinfo.h>
 #include <qtimer.h>
+#include <kstatusbar.h>
+#include <qfileinfo.h>
 
 #include "misc/decoder.h"
 #include "dom/dom2_range.h"
@@ -71,6 +89,7 @@
 #include "html/htmltokenizer.h"
 #include "khtml_part.h"
 #include "khtmlpart_p.h"
+#include <kparts/browserextension.h>
 
 #include "khtmlview.h"
 #include "rendering/render_replaced.h"
@@ -88,36 +107,43 @@ using namespace khtml;
 using namespace DOM;
 using namespace KJS;
 
-bool visual = false;
+static bool visual = false;
+static pid_t xvfb;
 
 // -------------------------------------------------------------------------
 
-int PartMonitor::sm_loopLevel = 0;
 PartMonitor *PartMonitor::sm_highestMonitor = NULL;
 
 PartMonitor::PartMonitor(KHTMLPart *_part)
 {
     m_part = _part;
     m_completed = false;
-    m_ownLoopLevel = 0;
     connect(m_part,SIGNAL(completed()),this,SLOT(partCompleted()));
     m_timer_waits = 200;
+    m_timeout_timer = new QTimer(this);
 }
 
+PartMonitor::~PartMonitor()
+{
+   if (this == sm_highestMonitor)
+	sm_highestMonitor = 0;
+}
+
+
 void PartMonitor::waitForCompletion()
 {
     if (!m_completed) {
-	m_ownLoopLevel = ++sm_loopLevel;
-	m_previousMonitor = sm_highestMonitor;
+
+        if (sm_highestMonitor)
+		return;
+
 	sm_highestMonitor = this;
 
         kapp->enter_loop();
 
-	if (--sm_loopLevel) {
-	    assert(m_previousMonitor);
-	    QTimer::singleShot( visual ? 100 : 10 , m_previousMonitor, SLOT( timeout() ) );
-	}
-	sm_highestMonitor = m_previousMonitor;
+        //connect(m_timeout_timer, SIGNAL(timeout()), this, SLOT( timeout() ) );
+        //m_timeout_timer->stop();
+	//m_timeout_timer->start( visual ? 100 : 2, true );
     }
 
     QTimer::singleShot( 0, this, SLOT( finishTimers() ) );
@@ -133,7 +159,7 @@ void PartMonitor::finishTimers()
 {
     KJS::Window *w = KJS::Window::retrieveWindow( m_part );
     --m_timer_waits;
-    if ( m_timer_waits && w && w->winq->hasTimers() ) {
+    if ( m_timer_waits && (w && w->winq->hasTimers()) || m_part->inProgress()) {
         // wait a bit
         QTimer::singleShot( 10, this, SLOT(finishTimers() ) );
         return;
@@ -143,18 +169,15 @@ void PartMonitor::finishTimers()
 
 void PartMonitor::partCompleted()
 {
-    if (m_ownLoopLevel == 0) {
-        m_completed = true;
-    }
-    else if (m_ownLoopLevel == sm_loopLevel)
-    {
-        RenderWidget::flushWidgetResizes();
-        QTimer::singleShot( visual ? 100 : 2, this, SLOT( timeout() ) );
-    }
+    m_completed = true;
+    RenderWidget::flushWidgetResizes();
+    m_timeout_timer->stop();
+    connect(m_timeout_timer, SIGNAL(timeout()),this, SLOT( timeout() ) );
+    m_timeout_timer->start( visual ? 100 : 2, true );
     disconnect(m_part,SIGNAL(completed()),this,SLOT(partCompleted()));
 }
 
-void signal_handler( int x )
+static void signal_handler( int )
 {
     printf( "timeout\n" );
     abort();
@@ -216,17 +239,23 @@ Value RegTestFunction::call(ExecState *exec, Object &/*thisObj*/, const List &ar
                 docimpl->view()->layout();
             }
             QString filename = args[0].toString(exec).qstring();
-	    filename = RegressionTest::curr->m_currentCategory+"/"+filename;
+            filename = RegressionTest::curr->m_currentCategory+"/"+filename;
+            int failures = RegressionTest::NoFailure;
             if ( m_regTest->m_genOutput ) {
-                m_regTest->reportResult( m_regTest->checkOutput(filename+"-dom"),
-                                         "Script-generated " + filename + "-dom");
-                m_regTest->reportResult( m_regTest->checkOutput(filename+"-render"),
-                                         "Script-generated " + filename + "-render");
+                if ( !m_regTest->reportResult( m_regTest->checkOutput(filename+"-dom"),
+                                               "Script-generated " + filename + "-dom") )
+                    failures |= RegressionTest::DomFailure;
+                if ( !m_regTest->reportResult( m_regTest->checkOutput(filename+"-render"),
+                                         "Script-generated " + filename + "-render") )
+                    failures |= RegressionTest::RenderFailure;
             } else {
                 // compare with output file
-                m_regTest->reportResult( m_regTest->checkOutput(filename+"-dom"), "DOM");
-                m_regTest->reportResult( m_regTest->checkOutput(filename+"-render"), "RENDER");
+                if ( !m_regTest->reportResult( m_regTest->checkOutput(filename+"-dom"), "DOM") )
+                    failures |= RegressionTest::DomFailure;
+                if ( !m_regTest->reportResult( m_regTest->checkOutput(filename+"-render"), "RENDER") )
+                    failures |= RegressionTest::RenderFailure;
             }
+            RegressionTest::curr->doFailureReport( filename, failures );
             break;
         }
         case Quit:
@@ -373,6 +402,8 @@ Value KHTMLPartFunction::call(ExecState *exec, Object &/*thisObj*/, const List &
 
 static KCmdLineOptions options[] =
 {
+    { "b", 0, 0 },
+    { "base <base_dir>", "Directory containing tests, basedir and output directories.", 0},
     { "d", 0, 0 },
     { "debug", "Do not supress debug output", 0},
     { "g", 0, 0 } ,
@@ -380,15 +411,24 @@ static KCmdLineOptions options[] =
     { "s", 0, 0 } ,
     { "show", "Show the window while running tests", 0 } ,
     { "t", 0, 0 } ,
-    { "test <filename>", "Run only a single test. Multiple options allowed.", 0 } ,
+    { "test <filename>", "Only run a single test. Multiple options allowed.", 0 } ,
     { "js",  "Only run .js tests", 0 },
     { "html", "Only run .html tests", 0},
+    { "noxvfb", "Do not use Xvfb", 0},
     { "o", 0, 0 },
     { "output <directory>", "Put output in <directory> instead of <base_dir>/output", 0 } ,
-    { "+base_dir", "Directory containing tests,basedir and output directories", 0 } ,
+    { "+[base_dir]", "Directory containing tests,basedir and output directories. Only regarded if -b is not specified.", 0 } ,
+    { "+[testcases]", "Relative path to testcase, or directory of testcases to be run (equivalent to -t).", 0 } ,
     KCmdLineLastOption
 };
 
+static bool existsDir(QCString dir)
+{
+    struct stat st;
+
+    return (!stat(dir.data(), &st) && S_ISDIR(st.st_mode));
+}
+
 int main(int argc, char *argv[])
 {
     // forget about any settings
@@ -397,7 +437,7 @@ int main(int argc, char *argv[])
         fprintf(stderr, "dang, I don't even know who I am.\n");
         exit(1);
     }
-        
+
     QString kh("/var/tmp/%1_non_existant");
     kh = kh.arg( pw->pw_name );
     setenv( "KDEHOME", kh.latin1(), 1 );
@@ -406,10 +446,71 @@ int main(int argc, char *argv[])
 
     signal( SIGALRM, signal_handler );
 
+    // workaround various Qt crashes by always enforcing a TrueColor visual
+    QApplication::setColorSpec( QApplication::ManyColor );
+
     KCmdLineArgs::init(argc, argv, "testregression", "TestRegression",
                        "Regression tester for khtml", "1.0");
     KCmdLineArgs::addCmdLineOptions(options);
 
+    KCmdLineArgs *args = KCmdLineArgs::parsedArgs( );
+
+    QCString baseDir = args->getOption("base");
+
+    if ( args->count() < 1 && baseDir.isEmpty() ) {
+	KCmdLineArgs::usage();
+	::exit( 1 );
+    }
+
+    int testcase_index = 0;
+    if (baseDir.isEmpty()) baseDir = args->arg(testcase_index++);
+
+    QFileInfo bdInfo(baseDir);
+    baseDir = QFile::encodeName(bdInfo.absFilePath());
+
+    const char *subdirs[] = {"tests", "baseline", "output", "resources"};
+    for ( int i = 0; i < 3; i++ ) {
+        QFileInfo sourceDir(QFile::encodeName( baseDir ) + "/" + subdirs[i]);
+        if ( !sourceDir.exists() || !sourceDir.isDir() ) {
+            fprintf(stderr,"ERROR: Source directory \"%s/%s\": no such directory.\n", (const char *)baseDir, subdirs[i]);
+            exit(1);
+        }
+    }
+
+    if (args->isSet("xvfb"))
+    {
+        QString xvfbPath = KStandardDirs::findExe("Xvfb");
+        if ( xvfbPath.isEmpty() ) {
+            fprintf( stderr, "ERROR: We need Xvfb to be installed for reliable results\n" );
+            exit( 1 );
+        }
+        
+        QCString xvfbPath8 = QFile::encodeName(xvfbPath);
+        QStringList fpaths;
+        fpaths.append(baseDir+"/resources");
+
+        const char* const fontdirs[] = { "75dpi", "misc", "Type1" };
+        const char* const fontpaths[] =  {"/usr/share/fonts/", "/usr/X11/lib/X11/fonts/",
+            "/usr/lib/X11/fonts/", "/usr/share/fonts/X11/" };
+
+        for (size_t fp=0; fp < sizeof(fontpaths)/sizeof(*fontpaths); ++fp)
+            for (size_t fd=0; fd < sizeof(fontdirs)/sizeof(*fontdirs); ++fd)
+                if (existsDir(QCString(fontpaths[fp])+QCString(fontdirs[fd])))
+                    if (strcmp(fontdirs[fd] , "Type1"))
+                        fpaths.append(QCString(fontpaths[fp])+QCString(fontdirs[fd])+":unscaled");
+                    else
+                        fpaths.append(QCString(fontpaths[fp])+QCString(fontdirs[fd]));
+
+        xvfb = fork();
+        if ( !xvfb ) {
+            QCString buffer = fpaths.join(",").latin1();
+            execl( xvfbPath8.data(), xvfbPath8.data(), "-once", "-dpi", "100", "-screen", "0",
+                    "1024x768x16", "-ac", "-fp", buffer.data(), ":47", (char*)NULL );
+        }
+
+        setenv( "DISPLAY", ":47", 1 );
+    }
+
     KApplication a;
     a.disableAutoDcopRegistration();
     a.setStyle( "windows" );
@@ -422,8 +523,8 @@ int main(int argc, char *argv[])
     a.config()->writeEntry("ReportJSErrors", false);
     KConfig cfg( "khtmlrc" );
     cfg.setGroup("HTML Settings");
-    cfg.writeEntry( "StandardFont", KGlobalSettings::generalFont().family() );
-    cfg.writeEntry( "FixedFont", KGlobalSettings::fixedFont().family() );
+    cfg.writeEntry( "StandardFont", HTML_DEFAULT_VIEW_SANSSERIF_FONT );
+    cfg.writeEntry( "FixedFont", HTML_DEFAULT_VIEW_FIXED_FONT );
     cfg.writeEntry( "SerifFont", HTML_DEFAULT_VIEW_SERIF_FONT );
     cfg.writeEntry( "SansSerifFont", HTML_DEFAULT_VIEW_SANSSERIF_FONT );
     cfg.writeEntry( "CursiveFont", HTML_DEFAULT_VIEW_CURSIVE_FONT );
@@ -432,16 +533,13 @@ int main(int argc, char *argv[])
     cfg.writeEntry( "MediumFontSize", 10 );
     cfg.writeEntry( "Fonts", QStringList() );
     cfg.writeEntry( "DefaultEncoding", "" );
+    cfg.setGroup("Java/JavaScript Settings");
+    cfg.writeEntry( "WindowOpenPolicy", KHTMLSettings::KJSWindowOpenAllow);
+
     cfg.sync();
 
-    KCmdLineArgs *args = KCmdLineArgs::parsedArgs( );
     int rv = 1;
 
-    if ( args->count() < 1 ) {
-	KCmdLineArgs::usage();
-	::exit( 1 );
-    }
-
     if ( !args->isSet( "debug" ) ) {
         KSimpleConfig dc( "kdebugrc" );
         static int areas[] = { 1000, 6000, 6005, 6010, 6020, 6030,
@@ -458,15 +556,6 @@ int main(int argc, char *argv[])
         kdClearDebugConfig();
     }
 
-    const char *subdirs[] = {"tests", "baseline", "output"};
-    for ( int i = 0; i < 3; i++ ) {
-        QFileInfo sourceDir(QFile::encodeName( args->arg(0) ) + "/" + subdirs[i]);
-        if ( !sourceDir.exists() || !sourceDir.isDir() ) {
-            fprintf(stderr,"ERROR: Source directory \"%s/%s\": no such directory.\n",args->arg(0), subdirs[i]);
-            exit(1);
-        }
-    }
-
     // create widgets
     KHTMLFactory *fac = new KHTMLFactory();
     KMainWindow *toplevel = new KMainWindow();
@@ -486,6 +575,9 @@ int main(int argc, char *argv[])
     if ( visual )
         toplevel->show();
 
+    // we're not interested
+    toplevel->statusBar()->hide();
+
     if (!getenv("KDE_DEBUG")) {
         // set ulimits
         rlimit vmem_limit = { 256*1024*1024, RLIM_INFINITY };	// 256Mb Memory should suffice
@@ -496,7 +588,7 @@ int main(int argc, char *argv[])
 
     // run the tests
     RegressionTest *regressionTest = new RegressionTest(part,
-                                                        args->arg(0),
+                                                        baseDir,
                                                         args->getOption("output"),
                                                         args->isSet("genoutput"),
                                                         !args->isSet( "html" ),
@@ -508,6 +600,9 @@ int main(int argc, char *argv[])
 
     bool result = false;
     QCStringList tests = args->getOptionList("test");
+    // merge testcases specified on command line
+    for (; testcase_index < args->count(); testcase_index++)
+        tests << args->arg(testcase_index);
     if (tests.count() > 0)
         for (QValueListConstIterator<QCString> it = tests.begin(); it != tests.end(); ++it) {
 	    result = regressionTest->runTests(*it,true);
@@ -566,6 +661,8 @@ int main(int argc, char *argv[])
     khtml::CSSStyleSelector::clear();
     khtml::RenderStyle::cleanup();
 
+    kill( xvfb, SIGINT );
+
     return rv;
 }
 
@@ -699,9 +796,11 @@ bool RegressionTest::runTests(QString relPath, bool mustExist, int known_failure
 	m_currentCategory = relativeDir;
 	m_currentTest = filename;
         m_known_failures = known_failure;
-	if ( filename.endsWith(".html") || filename.endsWith( ".htm" ) || filename.endsWith( ".xhtml" ) ) {
+	if ( filename.endsWith(".html") || filename.endsWith( ".htm" ) || filename.endsWith( ".xhtml" ) || filename.endsWith( ".xml" ) ) {
             if ( relPath.startsWith( "domts/" ) && !m_runJS )
                 return true;
+	    if ( relPath.startsWith( "ecma/" ) && !m_runJS )
+	        return true;
             if ( m_runHTML )
                 testStaticFile(relPath);
 	}
@@ -1034,7 +1133,7 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
 
     // are blocking reads possible with KProcess?
     char pwd[PATH_MAX];
-    getcwd( pwd, PATH_MAX );
+    (void) getcwd( pwd, PATH_MAX );
     chdir( QFile::encodeName( m_baseDir ) );
 
     if ( failures & RenderFailure ) {
@@ -1136,11 +1235,14 @@ void RegressionTest::doFailureReport( const QString& test, int failures )
         cl += "<span id='b4' class='button' onclick='showRender();m(4)'>R-DIFF</span>&nbsp;\n";
     if ( domDiff.length() )
         cl += "<span id='b5' class='button' onclick='showDom();m(5);'>D-DIFF</span>&nbsp;\n";
-    cl += QString( "<a class=button href=\"%2\">HTML</a>&nbsp;"
-                   "<hr>"
+    // The test file always exists - except for checkOutput called from *.js files
+    if ( QFile::exists( m_baseDir + "/tests/"+ test ) )
+        cl += QString( "<a class=button href=\"%1\">HTML</a>&nbsp;" )
+              .arg( relpath+"/tests/"+test );
+
+    cl += QString( "<hr>"
                    "<img style='border: solid 5px gray' src=\"%1\" id='image'>" )
-          .arg( relpath+"/baseline/"+test+"-dump.png" )
-          .arg( relpath+"/tests/"+test );
+          .arg( relpath+"/baseline/"+test+"-dump.png" );
 
     cl += "<div id='render' class='diff'>" + renderDiff + "</div>";
     cl += "<div id='dom' class='diff'>" + domDiff + "</div>";
@@ -1154,6 +1256,12 @@ void RegressionTest::testStaticFile(const QString & filename)
 {
     qApp->mainWidget()->resize( 800, 600); // restore size
 
+    // Set arguments
+    KParts::URLArgs args;
+    if (filename.endsWith(".html") || filename.endsWith(".htm")) args.serviceType = "text/html";
+    else if (filename.endsWith(".xhtml")) args.serviceType = "application/xhtml+xml";
+    else if (filename.endsWith(".xml")) args.serviceType = "text/xml";
+    m_part->browserExtension()->setURLArgs(args);
     // load page
     KURL url;
     url.setProtocol("file");
@@ -1201,6 +1309,9 @@ void RegressionTest::testStaticFile(const QString & filename)
                 description = comp2.value().toString( exec ).qstring();
         }
         reportResult( success,  description );
+
+        if (!success && !m_known_failures)
+            doFailureReport( filename, JSFailure );
         return;
     }
 
@@ -1234,9 +1345,7 @@ void RegressionTest::testStaticFile(const QString & filename)
 
         if ( m_known_failures & PaintFailure )
             m_known_failures = AllFailure;
-        CheckResult dumped = checkPaintdump(filename);
-        reportResult( dumped, "PAINT");
-        if (dumped == Failure)
+        if (!reportResult( checkPaintdump(filename), "PAINT") )
             failures |= PaintFailure;
 
         doFailureReport(filename, failures );
@@ -1324,7 +1433,7 @@ RegressionTest::CheckResult RegressionTest::checkPaintdump(const QString &filena
 {
     QString againstFilename( filename + "-dump.png" );
     QString absFilename = QFileInfo(m_baseDir + "/baseline/" + againstFilename).absFilePath();
-    if ( cvsIgnored( absFilename ) ) {
+    if ( svnIgnored( absFilename ) ) {
         m_known_failures = NoFailure;
         return Ignored;
     }
@@ -1336,6 +1445,15 @@ RegressionTest::CheckResult RegressionTest::checkPaintdump(const QString &filena
     if ( !imageEqual( baseline, output ) ) {
         QString outputFilename = m_outputDir + "/" + againstFilename;
         createMissingDirs(outputFilename );
+
+        bool kf = false;
+        if ( m_known_failures & AllFailure )
+            kf = true;
+        else if ( m_known_failures & PaintFailure )
+            kf = true;
+        if ( kf )
+            outputFilename += "-KF";
+
         output.save(outputFilename, "PNG", 60);
     }
     else {
@@ -1348,7 +1466,7 @@ RegressionTest::CheckResult RegressionTest::checkPaintdump(const QString &filena
 RegressionTest::CheckResult RegressionTest::checkOutput(const QString &againstFilename)
 {
     QString absFilename = QFileInfo(m_baseDir + "/baseline/" + againstFilename).absFilePath();
-    if ( cvsIgnored( absFilename ) ) {
+    if ( svnIgnored( absFilename ) ) {
         m_known_failures = NoFailure;
         return Ignored;
     }
@@ -1498,10 +1616,10 @@ void RegressionTest::slotOpenURL(const KURL &url, const KParts::URLArgs &args)
     pm.waitForCompletion();
 }
 
-bool RegressionTest::cvsIgnored( const QString &filename )
+bool RegressionTest::svnIgnored( const QString &filename )
 {
     QFileInfo fi( filename );
-    QString ignoreFilename = fi.dirPath() + "/.cvsignore";
+    QString ignoreFilename = fi.dirPath() + "/svnignore";
     QFile ignoreFile(ignoreFilename);
     if (!ignoreFile.open(IO_ReadOnly))
         return false;
