@@ -18,10 +18,9 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
- * $Id: dom2_rangeimpl.h,v 1.21 2003/10/07 19:31:47 savernik Exp $
  */
 
 #ifndef _DOM2_RangeImpl_h_
@@ -36,14 +35,13 @@ class RangeImpl : public khtml::Shared<RangeImpl>
 {
     friend class DocumentImpl;
 public:
-    RangeImpl(DocumentPtr *_ownerDocument);
-    RangeImpl(DocumentPtr *_ownerDocument,
+    RangeImpl(DocumentImpl *_ownerDocument);
+    RangeImpl(DocumentImpl *_ownerDocument,
               NodeImpl *_startContainer, long _startOffset,
               NodeImpl *_endContainer, long _endOffset);
 
     ~RangeImpl();
 
-    // ### remove the get from these methods (i.e. getStartContainer() -> startContainer())
     NodeImpl *startContainer(int &exceptioncode) const;
     long startOffset(int &exceptioncode) const;
     NodeImpl *endContainer(int &exceptioncode) const;
@@ -63,7 +61,25 @@ public:
     DocumentFragmentImpl *cloneContents ( int &exceptioncode );
     void insertNode( NodeImpl *newNode, int &exceptioncode );
     DOMString toString ( int &exceptioncode );
-    DOMString toHTML (  );
+    /** Converts the selection  to HTML.  The returned string will have matching
+     *  tags, and all td, tr, etc tags will be inside a table tag.  CSS is not
+     *  used at this stage - This needs to be fixed.
+     *
+     *  This is guaranteed to produce an xml valid snippet, no matter how crappy the input
+     *  html page is.  It will have html and body tags.
+     *  
+     *  Any urls in images or links will be expanded to full urls <em>with passwords stripped</em>
+     *  for security reasons.
+     *
+     *  Note: Originally this function didn't have the exceptioncode argument.  I added it
+     *  since all the other functions do.  If this is correct, please remove this comment.
+     *
+     *  @param exceptioncode This will be set if m_detached is true.
+     *  @return A string with html tags for this range.
+     *
+     *  @since 3.4
+     */
+    DOMString toHTML ( int &exceptioncode );
 
     DocumentFragment createContextualFragment ( const DOMString &html, int &exceptioncode );
 
@@ -89,7 +105,7 @@ public:
     bool readOnly() { return false; }
 
 protected:
-    DocumentPtr *m_ownerDocument;
+    DocumentImpl *m_ownerDocument;
     NodeImpl *m_startContainer;
     unsigned long m_startOffset;
     NodeImpl *m_endContainer;
