@@ -763,6 +763,14 @@ unsigned short DocumentImpl::nodeType() const
     return Node::DOCUMENT_NODE;
 }
 
+DOMStringImpl* DocumentImpl::textContent() const
+{
+    return 0;
+}
+
+void DocumentImpl::setTextContent( const DOMString&, int& )
+{}
+
 ElementImpl *DocumentImpl::createHTMLElement( const DOMString &name )
 {
     uint id = khtml::getTagID( name.string().lower().latin1(), name.string().length() );
@@ -2498,7 +2506,7 @@ void DocumentImpl::error(int err, const QString &text)
 void DocumentImpl::defaultEventHandler(EventImpl *evt)
 {
     // if any html event listeners are registered on the window, then dispatch them here
-    if (!m_windowEventListeners.listeners)
+    if (!m_windowEventListeners.listeners || evt->propagationStopped())
         return;
 
     QValueList<RegisteredEventListener>::iterator it;
@@ -2855,6 +2863,14 @@ NodeImpl *DocumentTypeImpl::cloneNode ( bool /*deep*/ )
     return 0;
 }
 
+DOMStringImpl* DocumentTypeImpl::textContent() const
+{
+    return 0;
+}
+    
+void DocumentTypeImpl::setTextContent( const DOMString&, int& )
+{}
+
 NamedNodeMapImpl * DocumentTypeImpl::entities() const
 {
     if ( !m_entities ) {
