@@ -14,8 +14,8 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include <unistd.h>
@@ -47,9 +47,11 @@
 #include <kio/jobclasses.h>
 #include <kio/netaccess.h>
 #include <kio/previewjob.h>
+#include <kio/renamedlg.h>
 #include <kpropertiesdialog.h>
 #include <kservicetypefactory.h>
 #include <kstdaccel.h>
+#include <kde_file.h>
 
 #include "config-kfile.h"
 #include "kcombiview.h"
@@ -244,6 +246,7 @@ void KDirOperator::insertViewDependentActions()
 
 void KDirOperator::activatedMenu( const KFileItem *, const QPoint& pos )
 {
+    setupMenu();
     updateSelectionDependentActions();
 
     actionMenu->popup( pos );
@@ -253,6 +256,7 @@ void KDirOperator::updateSelectionDependentActions()
 {
     bool hasSelection = m_fileView && m_fileView->selectedItems() &&
                         !m_fileView->selectedItems()->isEmpty();
+    myActionCollection->action( "trash" )->setEnabled( hasSelection );
     myActionCollection->action( "delete" )->setEnabled( hasSelection );
     myActionCollection->action( "properties" )->setEnabled( hasSelection );
 }
@@ -381,10 +385,14 @@ void KDirOperator::slotToggleIgnoreCase()
 void KDirOperator::mkdir()
 {
     bool ok;
-    QString where = url().prettyURL( +1, KURL::StripFileProtocol );
+    QString where = url().pathOrURL();
+    QString name = i18n( "New Folder" );
+    if ( url().isLocalFile() && QFileInfo( url().path(+1) + name ).exists() )
+         name = KIO::RenameDlg::suggestName( url(), name );
+
     QString dir = KInputDialog::getText( i18n( "New Folder" ),
                                          i18n( "Create new folder in:\n%1" ).arg( where ),
-                                         i18n("New Folder"), &ok, this);
+                                         name, &ok, this);
     if (ok)
       mkdir( KIO::encodeFileName( dir ), true );
 }
@@ -411,8 +419,7 @@ bool KDirOperator::mkdir( const QString& directory, bool enterDirectory )
 
     if ( exists ) // url was already existant
     {
-        QString which = url.isLocalFile() ? url.path() : url.prettyURL();
-        KMessageBox::sorry(viewWidget(), i18n("A file or folder named %1 already exists.").arg(which));
+        KMessageBox::sorry(viewWidget(), i18n("A file or folder named %1 already exists.").arg(url.pathOrURL()));
         enterDirectory = false;
     }
     else if ( !writeOk ) {
@@ -468,10 +475,10 @@ KIO::DeleteJob * KDirOperator::del( const KFileItemList& items,
         }
         else
             ret = KMessageBox::warningContinueCancelList( parent,
-                i18n("translators: not called for n == 1", "Do you really want to delete these %n items?", items.count() ),
+                i18n("Do you really want to delete this item?", "Do you really want to delete these %n items?", items.count() ),
                                                     files,
                                                     i18n("Delete Files"),
-                                                    KGuiItem(i18n("Delete"), "editdelete"), "AskForDelete" );
+                                                    KStdGuiItem::del(), "AskForDelete" );
         doIt = (ret == KMessageBox::Continue);
     }
 
@@ -495,6 +502,74 @@ void KDirOperator::deleteSelected()
         del( *list );
 }
 
+KIO::CopyJob * KDirOperator::trash( const KFileItemList& items,
+                                    QWidget *parent,
+                                    bool ask, bool showProgress )
+{
+    if ( items.isEmpty() ) {
+        KMessageBox::information( parent,
+                                i18n("You did not select a file to trash."),
+                                i18n("Nothing to Trash") );
+        return 0L;
+    }
+
+    KURL::List urls;
+    QStringList files;
+    KFileItemListIterator it( items );
+
+    for ( ; it.current(); ++it ) {
+        KURL url = (*it)->url();
+        urls.append( url );
+        if ( url.isLocalFile() )
+            files.append( url.path() );
+        else
+            files.append( url.prettyURL() );
+    }
+
+    bool doIt = !ask;
+    if ( ask ) {
+        int ret;
+        if ( items.count() == 1 ) {
+            ret = KMessageBox::warningContinueCancel( parent,
+                i18n( "<qt>Do you really want to trash\n <b>'%1'</b>?</qt>" )
+                .arg( files.first() ),
+                                                      i18n("Trash File"),
+                                                      KGuiItem(i18n("to trash", "&Trash"),"edittrash"), "AskForTrash" );
+        }
+        else
+            ret = KMessageBox::warningContinueCancelList( parent,
+                i18n("translators: not called for n == 1", "Do you really want to trash these %n items?", items.count() ),
+                                                    files,
+                                                    i18n("Trash Files"),
+                                                    KGuiItem(i18n("to trash", "&Trash"),"edittrash"), "AskForTrash" );
+        doIt = (ret == KMessageBox::Continue);
+    }
+
+    if ( doIt ) {
+        KIO::CopyJob *job = KIO::trash( urls, showProgress );
+        job->setWindow (topLevelWidget());
+        job->setAutoErrorHandlingEnabled( true, parent );
+        return job;
+    }
+
+    return 0L;
+}
+
+void KDirOperator::trashSelected(KAction::ActivationReason reason, Qt::ButtonState state)
+{
+    if ( !m_fileView )
+        return;
+
+    if ( reason == KAction::PopupMenuActivation && ( state & Qt::ShiftButton ) ) {
+        deleteSelected();
+	return;
+    }
+
+    const KFileItemList *list = m_fileView->selectedItems();
+    if ( list )
+        trash( *list, this );
+}
+
 void KDirOperator::close()
 {
     resetCursor();
@@ -603,7 +678,7 @@ void KDirOperator::setURL(const KURL& _newurl, bool clearforward)
     backAction->setEnabled( !backStack.isEmpty() );
     upAction->setEnabled( !isRoot() );
 
-    dir->openURL( newurl );
+    openURL( newurl );
 }
 
 void KDirOperator::updateDir()
@@ -616,7 +691,17 @@ void KDirOperator::updateDir()
 void KDirOperator::rereadDir()
 {
     pathChanged();
-    dir->openURL( currUrl, false, true );
+    openURL( currUrl, false, true );
+}
+
+
+bool KDirOperator::openURL( const KURL& url, bool keep, bool reload )
+{
+    bool result = dir->openURL( url, keep, reload );
+    if ( !result ) // in that case, neither completed() nor canceled() will be emitted by KDL
+        slotCanceled();
+
+    return result;
 }
 
 // Protected
@@ -691,7 +776,7 @@ KURL KDirOperator::url() const
 
 void KDirOperator::cdUp()
 {
-    KURL tmp( currUrl );
+    KURL tmp(currUrl);
     tmp.cd(QString::fromLatin1(".."));
     setURL(tmp, true);
 }
@@ -987,7 +1072,7 @@ void KDirOperator::connectView(KFileView *view)
 
     if ( listDir ) {
         QApplication::setOverrideCursor( waitCursor );
-        dir->openURL( currUrl );
+        openURL( currUrl );
     }
     else
         view->listingCompleted();
@@ -1168,7 +1253,8 @@ void KDirOperator::slotCompletionMatch(const QString& match)
 
 void KDirOperator::setupActions()
 {
-    myActionCollection = new KActionCollection( this, "KDirOperator::myActionCollection" );
+    myActionCollection = new KActionCollection( topLevelWidget(), this, "KDirOperator::myActionCollection" );
+
     actionMenu = new KActionMenu( i18n("Menu"), myActionCollection, "popupMenu" );
     upAction = KStdAction::up( this, SLOT( cdUp() ), myActionCollection, "up" );
     upAction->setText( i18n("Parent Folder") );
@@ -1182,7 +1268,10 @@ void KDirOperator::setupActions()
                                                    "viewActionSeparator" );
     mkdirAction = new KAction( i18n("New Folder..."), 0,
                                  this, SLOT( mkdir() ), myActionCollection, "mkdir" );
-    new KAction( i18n( "Delete" ), "editdelete", Key_Delete, this,
+    KAction* trash = new KAction( i18n( "Move to Trash" ), "edittrash", Key_Delete, myActionCollection, "trash" );
+    connect( trash, SIGNAL( activated( KAction::ActivationReason, Qt::ButtonState ) ),
+	     this, SLOT( trashSelected( KAction::ActivationReason, Qt::ButtonState ) ) );
+    new KAction( i18n( "Delete" ), "editdelete", SHIFT+Key_Delete, this,
                   SLOT( deleteSelected() ), myActionCollection, "delete" );
     mkdirAction->setIcon( QString::fromLatin1("folder_new") );
     reloadAction->setText( i18n("Reload") );
@@ -1294,7 +1383,13 @@ void KDirOperator::setupMenu(int whichActions)
     if (whichActions & FileActions)
     {
         actionMenu->insert( mkdirAction );
-        actionMenu->insert( myActionCollection->action( "delete" ) );
+        if (currUrl.isLocalFile() && !(KApplication::keyboardMouseState() & Qt::ShiftButton))
+            actionMenu->insert( myActionCollection->action( "trash" ) );
+        KConfig *globalconfig = KGlobal::config();
+        KConfigGroupSaver cs( globalconfig, QString::fromLatin1("KDE") );
+        if (!currUrl.isLocalFile() || (KApplication::keyboardMouseState() & Qt::ShiftButton) ||
+            globalconfig->readBoolEntry("ShowDeleteCommand", false))
+            actionMenu->insert( myActionCollection->action( "delete" ) );
         actionMenu->insert( actionSeparator );
     }
 
@@ -1592,9 +1687,9 @@ bool KDirOperator::isReadable( const KURL& url )
     if ( !url.isLocalFile() )
 	return true; // what else can we say?
 
-    struct stat buf;
+    KDE_struct_stat buf;
     QString ts = url.path(+1);
-    bool readable = ( ::stat( QFile::encodeName( ts ), &buf) == 0 );
+    bool readable = ( KDE_stat( QFile::encodeName( ts ), &buf) == 0 );
     if (readable) { // further checks
 	DIR *test;
 	test = opendir( QFile::encodeName( ts )); // we do it just to test here
