@@ -193,7 +193,10 @@ Q_LONG KTar::readRawHeader(char *buffer) {
 
       // only compare those of the 6 checksum digits that mean something,
       // because the other digits are filled with all sorts of different chars by different tars ...
-      if( strncmp( buffer + 148 + 6 - s.length(), s.data(), s.length() ) ) {
+      // Some tars right-justify the checksum so it could start in one of three places - we have to check each.
+      if( strncmp( buffer + 148 + 6 - s.length(), s.data(), s.length() )
+        && strncmp( buffer + 148 + 7 - s.length(), s.data(), s.length() )
+        && strncmp( buffer + 148 + 8 - s.length(), s.data(), s.length() ) ) {
         kdWarning(7041) << "KTar: invalid TAR file. Header is: " << QCString( buffer+257, 5 ) << endl;
         return -1;
       }
@@ -301,10 +304,10 @@ bool KTar::KTarPrivate::fillTempFile( const QString & filename) {
             delete filterDev;
             return false;
         }
-        Q_LONG len;
-        while ( !filterDev->atEnd() ) {
+        Q_LONG len = -1;
+        while ( !filterDev->atEnd() && len != 0) {
             len = filterDev->readBlock(buffer.data(),buffer.size());
-            if ( len <= 0 ) { // corrupted archive
+            if ( len < 0 ) { // corrupted archive
                 delete filterDev;
                 return false;
             }
@@ -436,8 +439,8 @@ bool KTar::openArchive( int mode )
                     // Let's hack around hard links. Our classes don't support that, so make them symlinks
                     if ( typeflag == '1' )
                     {
-                        size = nm.length(); // in any case, we don't want to skip the real size, hence this resetting of size
-                        kdDebug(7041) << "HARD LINK, setting size to " << size << endl;
+                        kdDebug(7041) << "HARD LINK, setting size to 0 instead of " << size << endl;
+                        size = 0; // no contents
                     }
 
                     //kdDebug(7041) << "KTar::openArchive file " << nm << " size=" << size << endl;
@@ -798,7 +801,7 @@ bool KTar::prepareWriting_impl(const QString &name, const QString &user,
 
     QCString permstr;
     permstr.sprintf("%o",perm);
-    permstr.rightJustify(6, ' ');
+    permstr = permstr.rightJustify(6, ' ');
     fillBuffer(buffer, permstr, size, mtime, 0x30, uname, gname);
 
     // Write header
@@ -857,7 +860,7 @@ bool KTar::writeDir_impl(const QString &name, const QString &user,
 
     QCString permstr;
     permstr.sprintf("%o",perm);
-    permstr.rightJustify(6, ' ');
+    permstr = permstr.rightJustify(6, ' ');
     fillBuffer( buffer, permstr, 0, mtime, 0x35, uname, gname);
 
     // Write header
@@ -921,7 +924,7 @@ bool KTar::writeSymLink_impl(const QString &name, const QString &target,
 
     QCString permstr;
     permstr.sprintf("%o",perm);
-    permstr.rightJustify(6, ' ');
+    permstr = permstr.rightJustify(6, ' ');
     fillBuffer(buffer, permstr, 0, mtime, 0x32, uname, gname);
 
     // Write header
