@@ -13,8 +13,8 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 /*
@@ -37,20 +37,22 @@
 
 */
 
+#include "kzip.h"
+#include "kfilterdev.h"
+#include "klimitediodevice.h"
+#include <kmimetype.h>
+#include <ksavefile.h>
+#include <kdebug.h>
+
 #include <qasciidict.h>
 #include <qfile.h>
 #include <qdir.h>
+#include <qdatetime.h>
+#include <qptrlist.h>
+
+#include <zlib.h>
 #include <time.h>
 #include <string.h>
-#include <qdatetime.h>
-#include <kdebug.h>
-#include <qptrlist.h>
-#include <kmimetype.h>
-#include <zlib.h>
-
-#include "kfilterdev.h"
-#include "kzip.h"
-#include "klimitediodevice.h"
 
 const int max_path_len = 4095;	// maximum number of character a path may contain
 
@@ -83,6 +85,24 @@ static void transformToMsDos(const QDateTime& dt, char* buffer)
     }
 }
 
+static time_t transformFromMsDos(const char* buffer)
+{
+    Q_UINT16 time = (uchar)buffer[0] | ( (uchar)buffer[1] << 8 );
+    int h = time >> 11;
+    int m = ( time & 0x7ff ) >> 5;
+    int s = ( time & 0x1f ) * 2 ;
+    QTime qt(h, m, s);
+
+    Q_UINT16 date = (uchar)buffer[2] | ( (uchar)buffer[3] << 8 );
+    int y = ( date >> 9 ) + 1980;
+    int o = ( date & 0x1ff ) >> 5;
+    int d = ( date & 0x1f );
+    QDate qd(y, o, d);
+
+    QDateTime dt( qd, qt );
+    return dt.toTime_t();
+}
+
 // == parsing routines for zip headers
 
 /** all relevant information about parsing file information */
@@ -126,16 +146,18 @@ static bool parseExtTimestamp(const char *buffer, int size, bool islocal,
   }/*end if*/
   int flags = *buffer;		// read flags
   buffer += 1;
+  size -= 1;
 
   if (flags & 1) {		// contains modification time
-    if (size < 5) {
+    if (size < 4) {
       kdDebug(7040) << "premature end of extended timestamp (#2)" << endl;
       return false;
     }/*end if*/
     pfi.mtime = time_t((uchar)buffer[0] | (uchar)buffer[1] << 8
     			| (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+    buffer += 4;
+    size -= 4;
   }/*end if*/
-  buffer += 4;
   // central extended field cannot contain more than the modification time
   // even if other flags are set
   if (!islocal) {
@@ -144,24 +166,25 @@ static bool parseExtTimestamp(const char *buffer, int size, bool islocal,
   }/*end if*/
 
   if (flags & 2) {		// contains last access time
-    if (size < 9) {
+    if (size < 4) {
       kdDebug(7040) << "premature end of extended timestamp (#3)" << endl;
-      return false;
+      return true;
     }/*end if*/
     pfi.atime = time_t((uchar)buffer[0] | (uchar)buffer[1] << 8
     			| (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+    buffer += 4;
+    size -= 4;
   }/*end if*/
-  buffer += 4;
 
   if (flags & 4) {		// contains creation time
-    if (size < 13) {
+    if (size < 4) {
       kdDebug(7040) << "premature end of extended timestamp (#4)" << endl;
-      return false;
+      return true;
     }/*end if*/
     pfi.ctime = time_t((uchar)buffer[0] | (uchar)buffer[1] << 8
     			| (uchar)buffer[2] << 16 | (uchar)buffer[3] << 24);
+    buffer += 4;
   }/*end if*/
-  buffer += 4;
 
   pfi.exttimestamp_seen = true;
   return true;
@@ -293,7 +316,8 @@ public:
           m_currentDev( 0L ),
           m_compression( 8 ),
           m_extraField( KZip::NoExtraField ),
-	  m_offset( 0L ) { }
+	  m_offset( 0L ),
+          m_saveFile( 0 ) {}
 
     unsigned long           m_crc;         // checksum
     KZipFileEntry*          m_currentFile; // file currently being written
@@ -301,19 +325,23 @@ public:
     QPtrList<KZipFileEntry> m_fileList;    // flat list of all files, for the index (saves a recursive method ;)
     int                     m_compression;
     KZip::ExtraField        m_extraField;
-    unsigned int	        m_offset; // holds the offset of the place in the zip,
+    unsigned int            m_offset; // holds the offset of the place in the zip,
     // where new data can be appended. after openarchive it points to 0, when in
     // writeonly mode, or it points to the beginning of the central directory.
     // each call to writefile updates this value.
+    KSaveFile*              m_saveFile;
 };
 
 KZip::KZip( const QString& filename )
     : KArchive( 0L )
 {
     //kdDebug(7040) << "KZip(filename) reached." << endl;
+    Q_ASSERT( !filename.isEmpty() );
     m_filename = filename;
     d = new KZipPrivate;
-    setDevice( new QFile( filename ) );
+    // unusual: this ctor leaves the device set to 0.
+    // This is for the use of KSaveFile, see openArchive.
+    // KDE4: move KSaveFile support to base class, KArchive.
 }
 
 KZip::KZip( QIODevice * dev )
@@ -329,8 +357,12 @@ KZip::~KZip()
     //kdDebug(7040) << "~KZip reached." << endl;
     if( isOpened() )
         close();
-    if ( !m_filename.isEmpty() )
-        delete device(); // we created it ourselves
+    if ( !m_filename.isEmpty() ) { // we created the device ourselves
+        if ( d->m_saveFile ) // writing mode
+            delete d->m_saveFile;
+        else // reading mode
+            delete device(); // (the QFile)
+    }
     delete d;
 }
 
@@ -339,10 +371,35 @@ bool KZip::openArchive( int mode )
     //kdDebug(7040) << "openarchive reached." << endl;
     d->m_fileList.clear();
 
-    if ( mode == IO_WriteOnly )
+    switch ( mode ) {
+    case IO_WriteOnly:
+        // The use of KSaveFile can't be done in the ctor (no mode known yet)
+        // Ideally we would reimplement open() and do it there (BIC)
+        if ( !m_filename.isEmpty() ) {
+            kdDebug(7040) << "Writing to a file using KSaveFile" << endl;
+            d->m_saveFile = new KSaveFile( m_filename );
+            if ( d->m_saveFile->status() != 0 ) {
+                kdWarning(7040) << "KSaveFile creation for " << m_filename << " failed, " << strerror( d->m_saveFile->status() ) << endl;
+                delete d->m_saveFile;
+                d->m_saveFile = 0;
+                return false;
+            }
+            Q_ASSERT( d->m_saveFile->file() );
+            setDevice( d->m_saveFile->file() );
+        }
         return true;
-    if ( mode != IO_ReadOnly && mode != IO_ReadWrite )
+    case IO_ReadOnly:
+    case IO_ReadWrite:
     {
+        // ReadWrite mode still uses QFile for now; we'd need to copy to the tempfile, in fact.
+        if ( !m_filename.isEmpty() ) {
+            setDevice( new QFile( m_filename ) );
+            if ( !device()->open( mode ) )
+                return false;
+        }
+        break; // continued below
+    }
+    default:
         kdWarning(7040) << "Unsupported mode " << mode << endl;
         return false;
     }
@@ -350,9 +407,13 @@ bool KZip::openArchive( int mode )
     char buffer[47];
 
     // Check that it's a valid ZIP file
-    // KArchive::open() opened the underlying device already.
+    // the above code opened the underlying device already.
     QIODevice* dev = device();
 
+    if (!dev) {
+        return false;
+    }
+
     uint offset = 0; // holds offset, where we read
     int n;
 
@@ -360,8 +421,13 @@ bool KZip::openArchive( int mode )
     QAsciiDict<ParseFileInfo> pfi_map(1009, true /*case sensitive */, true /*copy keys*/);
     pfi_map.setAutoDelete(true);
 
+    // We set a bool for knowing if we are allowed to skip the start of the file
+    bool startOfFile = true;
+
     for (;;) // repeat until 'end of entries' signature is reached
     {
+kdDebug(7040) << "loop starts" << endl;
+kdDebug(7040) << "dev->at() now : " << dev->at() << endl;
         n = dev->readBlock( buffer, 4 );
 
         if (n < 4)
@@ -372,11 +438,18 @@ bool KZip::openArchive( int mode )
         }
 
         if ( !memcmp( buffer, "PK\5\6", 4 ) ) // 'end of entries'
-            break;
-
-        if ( !memcmp( buffer, "PK\3\4", 4 ) ) // local file header
         {
-            dev->at( dev->at() + 2 ); // skip 'version needed to extract'
+	    kdDebug(7040) << "PK56 found end of archive" << endl;
+            startOfFile = false;
+	    break;
+	}
+
+	if ( !memcmp( buffer, "PK\3\4", 4 ) ) // local file header
+        {
+	    kdDebug(7040) << "PK34 found local file header" << endl;
+            startOfFile = false;
+            // can this fail ???
+	    dev->at( dev->at() + 2 ); // skip 'version needed to extract'
 
 	    // read static header stuff
             n = dev->readBlock( buffer, 24 );
@@ -387,6 +460,8 @@ bool KZip::openArchive( int mode )
 
 	    int gpf = (uchar)buffer[0];	// "general purpose flag" not "general protection fault" ;-)
 	    int compression_mode = (uchar)buffer[2] | (uchar)buffer[3] << 8;
+	    time_t mtime = transformFromMsDos( buffer+4 );
+
 	    Q_LONG compr_size = (uchar)buffer[12] | (uchar)buffer[13] << 8
 	    			| (uchar)buffer[14] << 16 | (uchar)buffer[15] << 24;
 	    Q_LONG uncomp_size = (uchar)buffer[16] | (uchar)buffer[17] << 8
@@ -394,6 +469,13 @@ bool KZip::openArchive( int mode )
 	    int namelen = (uchar)buffer[20] | (uchar)buffer[21] << 8;
 	    int extralen = (uchar)buffer[22] | (uchar)buffer[23] << 8;
 
+	    kdDebug(7040) << "general purpose bit flag: " << gpf << endl;
+	    kdDebug(7040) << "compressed size: " << compr_size << endl;
+	    kdDebug(7040) << "uncompressed size: " << uncomp_size << endl;
+	    kdDebug(7040) << "namelen: " << namelen << endl;
+	    kdDebug(7040) << "extralen: " << extralen << endl;
+	    kdDebug(7040) << "archive size: " << dev->size() << endl;
+
 	    // read filename
 	    QCString filename(namelen + 1);
 	    n = dev->readBlock(filename.data(), namelen);
@@ -403,21 +485,36 @@ bool KZip::openArchive( int mode )
 	    }
 
 	    ParseFileInfo *pfi = new ParseFileInfo();
+	    pfi->mtime = mtime;
 	    pfi_map.insert(filename.data(), pfi);
 
-	    // read and parse extra field
+            // read and parse the beginning of the extra field,
+            // skip rest of extra field in case it is too long
+            unsigned int extraFieldEnd = dev->at() + extralen;
 	    pfi->extralen = extralen;
 	    int handledextralen = QMIN(extralen, (int)sizeof buffer);
+
+	    kdDebug(7040) << "handledextralen: " << handledextralen << endl;
+
 	    n = dev->readBlock(buffer, handledextralen);
 	    // no error msg necessary as we deliberately truncate the extra field
 	    if (!parseExtraField(buffer, handledextralen, true, *pfi))
+	    {
+	        kdWarning(7040) << "Invalid ZIP File. Broken ExtraField." << endl;
 	        return false;
+	    }
+
+            // jump to end of extra field
+            dev->at( extraFieldEnd );
 
 	    // we have to take care of the 'general purpose bit flag'.
             // if bit 3 is set, the header doesn't contain the length of
             // the file and we look for the signature 'PK\7\8'.
             if ( gpf & 8 )
             {
+	    	// here we have to read through the compressed data to find
+		// the next PKxx
+	        kdDebug(7040) << "trying to seek for next PK78" << endl;
                 bool foundSignature = false;
 
                 while (!foundSignature)
@@ -439,21 +536,41 @@ bool KZip::openArchive( int mode )
                         return false;
                     }
 
-                    if ( buffer[0] == 'K' && buffer[1] == 7 && buffer[2] == 8 )
+                    // we have to detect three magic tokens here:
+		    // PK34 for the next local header in case there is no data descriptor
+		    // PK12 for the central header in case there is no data descriptor
+		    // PK78 for the data descriptor in case it is following the compressed data
+
+		    if ( buffer[0] == 'K' && buffer[1] == 7 && buffer[2] == 8 )
                     {
                         foundSignature = true;
                         dev->at( dev->at() + 12 ); // skip the 'data_descriptor'
                     }
+		    else if ( ( buffer[0] == 'K' && buffer[1] == 1 && buffer[2] == 2 )
+		         || ( buffer[0] == 'K' && buffer[1] == 3 && buffer[2] == 4 ) )
+                    {
+                        foundSignature = true;
+                        dev->at( dev->at() - 4 ); // go back 4 bytes, so that the magic bytes can be found...
+                    }
+                    else if ( buffer[0] == 'P' || buffer[1] == 'P' || buffer[2] == 'P' )
+                    {
+                        // We have another P character so we must go back a little to check if it is a magic
+                        dev->at( dev->at() - 3 );
+                    }
+
                 }
             }
             else
             {
+	    	// here we skip the compressed data and jump to the next header
+	        kdDebug(7040) << "general purpose bit flag indicates, that local file header contains valid size" << endl;
 		// check if this could be a symbolic link
 		if (compression_mode == NoCompression
 	    		&& uncomp_size <= max_path_len
 			&& uncomp_size > 0) {
 		    // read content and store it
 		    pfi->guessed_symlink.resize(uncomp_size + 1);
+       	            kdDebug(7040) << "guessed symlink size: " << uncomp_size << endl;
 		    n = dev->readBlock(pfi->guessed_symlink.data(), uncomp_size);
 		    if (n < uncomp_size) {
 			kdWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#5)" << endl;
@@ -461,30 +578,94 @@ bool KZip::openArchive( int mode )
 		    }
 		} else {
 
-                    dev->at( dev->at() + compr_size );
+	            if ( compr_size > (Q_LONG)dev->size() )
+		    {
+		    	// here we cannot trust the compressed size, so scan through the compressed
+			// data to find the next header
+			bool foundSignature = false;
+
+			while (!foundSignature)
+			{
+				n = dev->readBlock( buffer, 1 );
+				if (n < 1)
+				{
+					kdWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#2)" << endl;
+					return false;
+				}
+
+				if ( buffer[0] != 'P' )
+					continue;
+
+				n = dev->readBlock( buffer, 3 );
+				if (n < 3)
+				{
+					kdWarning(7040) << "Invalid ZIP file. Unexpected end of file. (#3)" << endl;
+					return false;
+				}
+
+				// we have to detect three magic tokens here:
+				// PK34 for the next local header in case there is no data descriptor
+				// PK12 for the central header in case there is no data descriptor
+				// PK78 for the data descriptor in case it is following the compressed data
+
+				if ( buffer[0] == 'K' && buffer[1] == 7 && buffer[2] == 8 )
+				{
+					foundSignature = true;
+					dev->at( dev->at() + 12 ); // skip the 'data_descriptor'
+				}
+
+				if ( ( buffer[0] == 'K' && buffer[1] == 1 && buffer[2] == 2 )
+					|| ( buffer[0] == 'K' && buffer[1] == 3 && buffer[2] == 4 ) )
+				{
+					foundSignature = true;
+					dev->at( dev->at() - 4 );
+					// go back 4 bytes, so that the magic bytes can be found
+					// in the next cycle...
+				}
+			}
+		    }
+		    else
+		    {
+// 			kdDebug(7040) << "before interesting dev->at(): " << dev->at() << endl;
+			bool success;
+			success = dev->at( dev->at() + compr_size ); // can this fail ???
+/*			kdDebug(7040) << "after interesting dev->at(): " << dev->at() << endl;
+			if ( success )
+				kdDebug(7040) << "dev->at was successful... " << endl;
+			else
+				kdDebug(7040) << "dev->at failed... " << endl;*/
+		    }
+
 		}
-                // here we calculate the length of the file in the zip
+
+// not needed any more
+/*                // here we calculate the length of the file in the zip
                 // with headers and jump to the next header.
                 uint skip = compr_size + namelen + extralen;
-                offset += 30 + skip;
+                offset += 30 + skip;*/
             }
         }
         else if ( !memcmp( buffer, "PK\1\2", 4 ) ) // central block
         {
+	    kdDebug(7040) << "PK12 found central block" << endl;
+            startOfFile = false;
 
             // so we reached the central header at the end of the zip file
-		    // here we get all interesting data out of the central header
+            // here we get all interesting data out of the central header
             // of a file
             offset = dev->at() - 4;
 
             //set offset for appending new files
             if ( d->m_offset == 0L ) d->m_offset = offset;
 
-		    n = dev->readBlock( buffer + 4, 42 );
+            n = dev->readBlock( buffer + 4, 42 );
             if (n < 42) {
                 kdWarning(7040) << "Invalid ZIP file, central entry too short" << endl; // not long enough for valid entry
                 return false;
             }
+
+            //int gpf = (uchar)buffer[9] << 8 | (uchar)buffer[10];
+            //kdDebug() << "general purpose flag=" << gpf << endl;
             // length of the filename (well, pathname indeed)
             int namelen = (uchar)buffer[29] << 8 | (uchar)buffer[28];
             QCString bufferName( namelen + 1 );
@@ -616,6 +797,50 @@ bool KZip::openArchive( int mode )
             if ( !b )
               return false;
         }
+        else if ( startOfFile )
+        {
+            // The file does not start with any ZIP header (e.g. self-extractable ZIP files)
+            // Therefore we need to find the first PK\003\004 (local header)
+            kdDebug(7040) << "Try to skip start of file" << endl;
+            startOfFile = false;
+            bool foundSignature = false;
+
+            while (!foundSignature)
+            {
+                n = dev->readBlock( buffer, 1 );
+                if (n < 1)
+                {
+                    kdWarning(7040) << "Invalid ZIP file. Unexpected end of file. " << k_funcinfo << endl;
+                    return false;
+                }
+
+                if ( buffer[0] != 'P' )
+                    continue;
+
+                n = dev->readBlock( buffer, 3 );
+                if (n < 3)
+                {
+                    kdWarning(7040) << "Invalid ZIP file. Unexpected end of file. " << k_funcinfo << endl;
+                    return false;
+                }
+
+                // We have to detect the magic token for a local header: PK\003\004
+                /*
+                 * Note: we do not need to check the other magics, if the ZIP file has no
+                 * local header, then it has not any files!
+                 */
+                if ( buffer[0] == 'K' && buffer[1] == 3 && buffer[2] == 4 )
+                {
+                    foundSignature = true;
+                    dev->at( dev->at() - 4 ); // go back 4 bytes, so that the magic bytes can be found...
+                }
+                else if ( buffer[0] == 'P' || buffer[1] == 'P' || buffer[2] == 'P' )
+                {
+                        // We have another P character so we must go back a little to check if it is a magic
+                    dev->at( dev->at() - 3 );
+                }
+            }
+        }
         else
         {
             kdWarning(7040) << "Invalid ZIP file. Unrecognized header at offset " << offset << endl;
@@ -634,9 +859,14 @@ bool KZip::closeArchive()
         //kdDebug(7040) << "closearchive readonly reached." << endl;
         return true;
     }
+
+    kdDebug() << k_funcinfo << "device=" << device() << endl;
     //ReadWrite or WriteOnly
     //write all central dir file entries
 
+    if ( !device() ) // saving aborted
+        return false;
+
     // to be written at the end of the file...
     char buffer[ 22 ]; // first used for 12, then for 22 at the end
     uLong crc = crc32(0L, Z_NULL, 0);
@@ -810,7 +1040,14 @@ bool KZip::closeArchive()
     if ( device()->writeBlock( buffer, 22 ) != 22 )
         return false;
 
-    //kdDebug(7040) << "kzip.cpp reached." << endl;
+    if ( d->m_saveFile ) {
+        d->m_saveFile->close();
+        setDevice( 0 );
+        delete d->m_saveFile;
+        d->m_saveFile = 0;
+    }
+
+    //kdDebug(7040) << __FILE__" reached." << endl;
     return true;
 }
 
@@ -864,9 +1101,15 @@ bool KZip::prepareWriting_impl(const QString &name, const QString &user,
         return false;
     }
 
+    if ( !device() ) { // aborted
+        //kdWarning(7040) << "prepareWriting_impl: no device" << endl;
+        return false;
+    }
+
     // set right offset in zip.
     if ( !device()->at( d->m_offset ) ) {
         kdWarning(7040) << "prepareWriting_impl: cannot seek in ZIP file. Disk full?" << endl;
+        abort();
         return false;
     }
 
@@ -993,8 +1236,10 @@ bool KZip::prepareWriting_impl(const QString &name, const QString &user,
     delete[] buffer;
 
     Q_ASSERT( b );
-    if (!b)
+    if (!b) {
+        abort();
         return false;
+    }
 
     // Prepare device for writing the data
     // Either device() if no compression, or a KFilterDev to compress
@@ -1005,8 +1250,10 @@ bool KZip::prepareWriting_impl(const QString &name, const QString &user,
 
     d->m_currentDev = KFilterDev::device( device(), "application/x-gzip", false );
     Q_ASSERT( d->m_currentDev );
-    if ( !d->m_currentDev )
+    if ( !d->m_currentDev ) {
+        abort();
         return false; // ouch
+    }
     static_cast<KFilterDev *>(d->m_currentDev)->setSkipHeaders(); // Just zlib, not gzip
 
     b = d->m_currentDev->open( IO_WriteOnly );
@@ -1127,8 +1374,10 @@ bool KZip::writeData_impl(const char * c, uint i)
 {
     Q_ASSERT( d->m_currentFile );
     Q_ASSERT( d->m_currentDev );
-    if (!d->m_currentFile || !d->m_currentDev)
+    if (!d->m_currentFile || !d->m_currentDev) {
+        abort();
         return false;
+    }
 
     // crc to be calculated over uncompressed stuff...
     // and they didn't mention it in their docs...
@@ -1136,7 +1385,10 @@ bool KZip::writeData_impl(const char * c, uint i)
 
     Q_LONG written = d->m_currentDev->writeBlock( c, i );
     //kdDebug(7040) << "KZip::writeData wrote " << i << " bytes." << endl;
-    return written == (Q_LONG)i;
+    bool ok = written == (Q_LONG)i;
+    if ( !ok )
+        abort();
+    return ok;
 }
 
 void KZip::setCompression( Compression c )
@@ -1159,6 +1411,15 @@ KZip::ExtraField KZip::extraField() const
     return d->m_extraField;
 }
 
+void KZip::abort()
+{
+    if ( d->m_saveFile ) {
+        d->m_saveFile->abort();
+        setDevice( 0 );
+    }
+}
+
+
 ///////////////
 
 QByteArray KZipFileEntry::data() const
@@ -1197,4 +1458,3 @@ QIODevice* KZipFileEntry::device() const
               <<" please use a command-line tool to handle this file." << endl;
     return 0L;
 }
-
