@@ -20,7 +20,7 @@
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
 
-    $Id: player.cc 465272 2005-09-29 09:47:40Z mueller $
+    $Id: player.cc 539843 2006-05-11 20:08:40Z mueller $
 
     Send comments and bug fixes to Antonio Larrosa <larrosa@kde.org>
 
@@ -362,8 +362,8 @@ void MidiPlayer::parseSpecialEvents(void)
  
 #endif
                     strncpy(pspev->text,(char *)ev->data,
-                        (ev->length>1024)? (1023) : (ev->length) );
-                    pspev->text[(ev->length>1024)? (1023):(ev->length)]=0;
+                        (ev->length>= sizeof(lasttext))? sizeof(lasttext)-1 : (ev->length) );
+                    pspev->text[(ev->length>= sizeof(lasttext))? sizeof(lasttext)-1:(ev->length)]=0;
 #ifdef PLAYERDEBUG
                     printf("(%s)(%s)\n",pspev->text,lasttext);
 #endif
@@ -373,7 +373,7 @@ void MidiPlayer::parseSpecialEvents(void)
                       lasttexttime=pspev->absmilliseconds;
                       lasttexttype=pspev->type;
                       strncpy(lasttext, pspev->text, 1024);
-                      lasttext[1023] = 0;
+                      lasttext[sizeof(lasttext)-1] = 0;
 #endif
                       pspev->next=new SpecialEvent;
 #ifdef PLAYERDEBUG
